/* global assert:true */
module.exports = function (options) {
  describe('Adapter#find', function () {
    var adapter, User, Profile, Post, Comment

    beforeEach(function () {
      adapter = this.$$adapter
      User = this.$$User
      Profile = this.$$Profile
      Post = this.$$Post
      Comment = this.$$Comment
    })

    it('should exist', function () {
      assert.equal(typeof adapter.find, 'function', 'adapter should have a "find" method')
    })

    it('should find a user', async function () {
      var user = await adapter.create(User, {name: 'John'})
      var userId = user.id
      assert.equal(user.name, 'John')
      assert.isDefined(user.id)

      var user2 = await adapter.find(User, user.id)
      assert.equal(user2.name, 'John')
      assert.isDefined(user2.id)
      assert.equal(user2.id, userId)
      assert.equal(user2.name, 'John')

      var post = await adapter.create(Post, { content: 'test', userId: userId })
      var postId = post.id
      assert.equal(post.content, 'test')
      assert.isDefined(post.id)
      assert.isDefined(post.userId)

      var comments = await Promise.all([
        adapter.create(Comment, {
          content: 'test2',
          postId: post.id,
          userId: user.id
        }),
        adapter.create(Comment, {
          content: 'test3',
          postId: post.id,
          userId: user.id
        })
      ])

      comments.sort(function (a, b) {
        return a.content > b.content
      })

      var findPost = await adapter.find(Post, postId, {with: ['user', 'comment']})
      findPost.comments.sort(function (a, b) {
        return a.content > b.content
      })
      assert.objectsEqual(findPost.user, user)
      assert.objectsEqual(findPost.comments, comments)

      await adapter.destroyAll(Comment)
      await adapter.destroy(Post, postId)
      var destroyedUser = await adapter.destroy(User, userId)
      assert.equal(destroyedUser, userId)

      user = await adapter.find(User, userId)
      assert.isTrue(!user)
    })

    it('should load belongsTo relations', async function () {
      var profile = await adapter.create(Profile, { email: 'foo@test.com' })
      var user = await adapter.create(User, {name: 'John', profileId: profile.id})
      var post = await adapter.create(Post, {content: 'foo', userId: user.id})
      var comment = await adapter.create(Comment, { content: 'test2', postId: post.id, userId: post.userId })

      comment = await adapter.find(Comment, comment.id, {'with': ['user', 'user.profile', 'post', 'post.user']})
      assert.isDefined(comment)
      assert.isDefined(comment.post)
      assert.isDefined(comment.post.user)
      assert.isDefined(comment.user)
      assert.isDefined(comment.user.profile)
    })

    it('should load hasMany and belongsTo relations', async function () {
      var user = await adapter.create(User, { name: 'John' })
      await adapter.create(Profile, { email: 'foo@test.com', userId: user.id })
      var post = await adapter.create(Post, { content: 'foo', userId: user.id })
      await adapter.create(Comment, { content: 'test2', postId: post.id, userId: post.userId })

      var foundPost = await adapter.find(Post, post.id, {'with': ['user', 'comment', 'comment.user', 'comment.user.profile']})
      assert.isDefined(foundPost.comments)
      assert.isDefined(foundPost.comments[0].user)
      assert.isDefined(foundPost.comments[0].user.profile)
      assert.isDefined(foundPost.user)
    })
  })
}
